

function [P,n]=Lagrange(x,y,z)

%__________________________________________________________________________
%
% Funzione: Lagrange 
%___________________
%
% Chiamata:  [P,n]=Lagrange(x,y,z);
%
% La function 'Lagrange' calcola i valori assunti nei punti z(k), 
% k=1,...,m, m>=1, dati in ingresso, dal polinomio interpolante di 
% Lagrange di grado n, n>=0, costruito sui punti (x(i),y(i)), 
% i=1,...,n+1 anch'essi dati in ingresso.   
% 
%
% Parametri di input:
%____________________
%
% x,y     Vettori colonna di lunghezza n+1, contenenti, rispettivamente
%         le ascisse (nodi), che devono essere distinte (ossia x(i) deve
%         essere diverso da x(j) se i e' diverso da j) e le ordinate su 
%         cui si vuole costruire il polinomio di Lagrange di grado n.
% z       Vettore colonna di lunghezza m, contenente i punti in cui si 
%         vuole valutare il polinomio di Lagrange. 
% 
%
% Parametri di output:
%_____________________
%
% P       Matrice di dimensione m x 2, contenente, nella riga k-esima,il
%         punto z(k) e il valore assunto dal polinomio interpolante nel 
%         punto z(k).
% n       variabile intera contenente il grado del polinomio interpolante.
%         
%__________________________________________________________________________
%
% Esempio di utilizzo della function:
% ___________________________________
%
% clear all
% format short e
% 
% x=[0:0.2:0.8]';
% t=x;
% f='exp(t)';
% y=eval(f);
% 
% z=[x ; -0.4; 1.3;0.54; 0.45; 0.79;0.11];
% z=sort(z);
% 
% [P,n]=Lagrange(x,y,z)
%
%__________________________________________________________________________



n=length(x)-1;
m=length(z);

for i=1:n
    for j=i+1:n+1
        if abs(x(i)-x(j))<=1.e-14
            disp('I nodi devono essere distinti')
            return
        end
    end
end

for k=1:m
   for i=1:n+1
       xj=x;
       xj(i)=[];
       l(k,i)=prod(z(k)-xj)/prod(x(i)-xj);
   end
end
P=[z l*y];
