
function Gershgor(A,np)
%______________________________________________________________________________
%
% Function: Gershgor 
%___________________
%
% Chiamata della function:  Gershgor(A,np);
%
% La function 'Gershgor' disegna i cerchi, detti di Gershgorin, che consentono 
% la localizzazione degli autovalori.   
% 
%
% Parametri di input:
% ___________________
%
% A    matrice quadrata di cui vogliamo localizzare gli autovalori; 
% np   numero di punti che servono per tracciare le circonferenze.
%             
%
% Output:
% ________
%
% Figura MATLAB.        
%_____________________________________________________________________________
%
% Esempio di utilizzo della function:
% ___________________________________
%
% A= [-7 -9 9 ;11 13 -9; -16 -16 20];
% np=20;
% Gershgor(A,np)
%_____________________________________________________________________________
%

[n,m]=size(A);
if n~=m;
    disp('Matrice non quadrata')
    return
end

close all
t=0:2*pi/(np-1):2*pi;
rad=eig(A)+i*eps;              
dim=max(size(A));
xr=zeros(dim,np);yr=zeros(dim,np);
xc=zeros(dim,np);yc=zeros(dim,np);
rr=sum(abs(A'))-abs(diag(A))';  %Raggi dei cerchi per righe
rc=sum(abs(A))-abs(diag(A))';   %Raggi dei cerchi per colonne
%
% valutazione delle ascisse e ordinate che servono per tracciare i cerchi
% relativi alle somme per righe;
%
for k=1:dim,
   xr(k,:)=A(k,k)+rr(k)*cos(t);
   yr(k,:)=rr(k)*sin(t);
end
%
% valutazione delle ascisse e ordinate che servono per tracciare i cerchi
% relativi alle somme per colonne;
%
for k=1:dim,
   xc(k,:)=A(k,k)+rc(k)*cos(t);
   yc(k,:)=rc(k)*sin(t);
end
xmin=min([min(xr),min(xc)]);xmax=max([max(xr),max(xc)]);
ymin=min([min(yr),min(yc)]);ymax=max([max(yr),max(yc)]);
figure(1)
subplot(2,2,1)
plot(xr',yr')
title('Cerchi righe')
axis([xmin xmax ymin ymax]);grid
subplot(2,2,2)
plot(rad,'+m')  % se gli autovalori sono complessi questa istruzione grafica 
                % la parte reale verso la parte immaginaria dell'autovalore
grid
xlabel('+ radici')
subplot(2,2,3)
plot(xc',yc')
title('Cerchi colonne')
axis([xmin xmax ymin ymax]);grid
subplot(2,2,4)
plot(rad,'+m')
grid
xlabel('+ radici')

figure(2)
plot(xr',yr','--',xc',yc')
title('Cerchi di Gershoring: cerchi righe ''--'', cerchi colonne ''-''')
grid