
function [B,V,it]=Jacobi_classico(A,nmax,toll);
%_____________________________________________________________________________
%
% Function: Jacobi_classico
% __________________________
%
%
%    Chiamata:  [B,V,it]=Jacobi_classico(A,nmax,toll);
% 
%    Commento:
%
%    La function 'Jacobi_classico' calcola gli autovalori ed i corrispondenti 
%    autovettori di una matrice A reale e simmetrica mediante il metodo di 
%    Jacobi classico. 
%    Il test d'arresto e' dato da: max|Bij|<toll  con i<j.
%         
%
% Parametri di input:
% ___________________  
%
% A       matrice reale simmetrica, di dimensione n x n, di cui si cercano 
%         gli autovalori e i corrispondenti autovettori;
% nmax    numero massimo di iterazioni;
% toll    tolleranza per il test d'arresto.
%
% Parametri di output:
% ____________________
%
% B       matrice diagonale, nella tolleranza posta, che contiene le 
%         approssimazioni degli autovalori di A;
% V       matrice le cui colonne contengono gli autovettori corrispondenti
%         agli autovalori;
% it      numero di iterazioni effettuate dal metodo.
%_____________________________________________________________________________
%
% Esempio di utilizzo della function:
% ___________________________________
%
% A=[22 -.5 .5; -.5 23 0; .5 0 24.5];
% toll=1.e-15;
% nmax=60;
% [B,V,it]=Jacobi_classico(A,nmax,toll);
% 
%_____________________________________________________________________________
%

V=[];it=[];
B=A;
[n,m]=size(B);
if n~=m
    disp('Matrice non quadrata')
    return
end
if A ~= A'
    disp('Matrice non simmetrica')
    return
end


it=0;
V=eye(n);
extrad=max(max(abs(triu(B,1))));
[p,q]=find(abs(triu(B,1))==extrad);
if length(p)>1
    p=p(1);
    q=q(1);
end
while extrad >= toll & it < nmax 
    mu=(B(q,q)-B(p,p))/(2*B(p,q));
    if mu==0
        t=1;
    else
        t=1/(mu+sign(mu)*sqrt(1+mu^2));
    end
    c=1/(sqrt(1+t^2));
    s=c*t;
    Q=eye(n);
    Q(p,p)=c;
    Q(p,q)=s;
    Q(q,p)=-s;
    Q(q,q)=c;
    B=Q'*B*Q;
    V=V*Q;
    extrad=max(max(abs(triu(B,1))));
    [p,q]=find(abs(triu(B,1))==extrad);
    if length(p)>1
        p=p(1);
        q=q(1);
    end
    it=it+1;
end
str=[];
for i=1:n
    str=[str ' %15.10f '];
end
fprintf(' \n Numero di Iterazioni       : %5d \n',it);
fprintf('\n Matrice contenente gli autovalori: \n B = \n');
fprintf( [str '\n'],B');
fprintf('\n Matrice contenente gli autovettori: \n V = \n');
fprintf( [str '\n'],V');
if it==nmax
   disp('** il metodo non converge nel numero di azzeramenti fissato **');
end            