
function [lam,xvet,it,lamdiff,xdiff]=Potenze(A,alfa,nmax,toll,x0);
%___________________________________________________________________________________
%
% Function: Potenze
% ___________________
%
%
%
%    Chiamata:  [lam,xvet,it,lamdiff,xdiff]=Potenze(A,alfa,nmax,toll,x0);
% 
%    Commento:
%
%    La function 'Potenze' calcola l'autovalore di modulo massimo (autovalore 
%    dominante) di una matrice reale A, mediante il metodo delle potenze con 
%    shift. La function fornisce anche l'autovettore corrispondente.
%    L'approssimazione dell'autovalore viene calcolata mediante il quoziente
%    di Rayleigh e si utilizza la norma 2 di vettore.
%    Si presuppone che A abbia uno solo autovalore di modulo massimo.
%         
%
% Parametri di input:
% ___________________
%
%
% A       matrice, di dimensione n x n, di cui si vuole conoscere l'autovalore
%         di modulo massimo;
% alfa    parametro di shift;
% nmax    numero massimo di iterazioni;
% toll    tolleranza per il test d'arresto;
% x0      vettore colonna di lunghezza n contenente l'approssimazione iniziale 
%         dell'autovettore.
%
% Parametri di output:
% ____________________
%
% lam     vettore colonna  di lunghezza it, contenente le approssimazioni 
%         dell'autovalore di modulo massimo calcolate (l'ultima componente
%          la soluzione con la tolleranza  fissata);
% xvet    matrice, di dimensioni it x n, contenente sulla i-esima riga 
%         l'approssimazione alla iterata i-esima dell' autovettore corrispondente
%         all'autovalore di modulo massimo;
% it      numero di iterazioni effettuate dal metodo;
% lamdiff vettore colonna di lunghezza it, contenente il  modulo della differenza
%         lam(i) - lam(i-1) di due approssimazioni successive dell'autovalore di 
%         modulo massimo;
% xdiff   vettore colonna di lunghezza it, contenente la norma 2 della differenza
%         xvet(i) - xvet(i-1), di due approssimazioni successive dell'autovettore 
%         dominante.
%___________________________________________________________________________________
%
% Esempio di utilizzo della function:
% ___________________________________
%
% A=[4 3 2 1; 3 4 3 2 ; 2 3 4 3; 1 2 3 4];
% alfa=0;
% nmax=60;
% toll=1.e-14;
% x0=[0 1 0 0]';
% [lam,xvet,it,lamdiff,xdiff]=Potenze(A,alfa,nmax,toll,x0);
%___________________________________________________________________________________
%

lam=[];
xvet=[];
lamdiff=[];
xdiff=[];
A=A-alfa*eye(size(A));
lam0=alfa;
x0=x0/norm(x0);
[n,m]=size(A);
if n~=m
   disp('***ERRORE*** matrice non quadrata')
   return
end
err=1;
it=0;
while err > toll    & it < nmax 
    x1=A*x0;
    lam1=x0'*x1;
    x1=x1/norm(x1);
    err=abs(lam1-lam0);
    err_x=norm(x1-x0);
    xdiff=[xdiff; err_x];
    lamdiff=[lamdiff;err];
    lam=[lam;lam1+alfa];
    xvet=[xvet; x1'];
    x0=x1;
    lam0=lam1;
    it=it+1;
end
fprintf(' \n Numero di Iterazioni       : %5d \n',it);
fprintf(' Autovalore di modulo massimo:  %22.12e \n',lam(it));
fprintf(' Autovettore corrispondente : \n');
disp(x1');
if it==nmax
   disp('** il metodo non converge nel numero di iterazioni fissato **');
end