
function lam=Tridieig(d,b,ind,toll,nmax)

%_____________________________________________________________________________
%
% Function: Tridieig
% ___________________
%
%
%    Chiamata:  lam=Tridieig(d,b,ind,toll,nmax);
% 
%    Commento:
%
%    La function 'Tridieig' implementa il metodo della bisezione per il calcolo 
%    degli autovalori di una matrice A tridiagonale simmetrica, di dimensioni 
%    n x n. 
%    Una matrice A, simmetrica, pu essere trasformata in una tridiagonale 
%    mediante il comando MATLAB 'hess(A)'.
%         
%
% Parametri di input:
% ___________________
%
% d     vettore colonna, di lunghezza n, contenente la diagonale principale 
%       di A;
% b     vettore colonna, di lunghezza (n-1), contenente la codiagonale di A
%       (sopradiagonale = sottodiagonale);
% ind   vettore riga contenente gli indici degli autovalori di A, che vogliamo 
%       calcolare;
% nmax  numero massimo di iterazioni;
% toll  tolleranza per il test d'arresto.
%
% Parametri di output:
% ____________________
%
%  lam   vettore colonna contenente gli autovalori di A richiesti posti in 
%        ordine di indice crescente.
%_____________________________________________________________________________
%
% Esempio di utilizzo della function:
% ___________________________________
%
% A=[4 3 2 1; 3 4 3 2 ; 2 3 4 3; 1 2 3 4];
% A1=hess(A);    %calcolo della matrice tridiagonale mediante la trasformazione 
%                %in Hessenberg superiore 
% d=diag(A1);    
% b=diag(A1,-1);
% ind=[ 1 3 4];
% toll=1.e-15;
% nmax=100;
% lam=Tridieig(d,b,ind,toll,nmax);
%_____________________________________________________________________________
%

b=[0; b];
n=length(d);
m=length(b);
lam=[];
if m~=n
    disp('Vettore codiagonale non congruente con la dimensione di A')
    return
end

if b==0 
    disp(' Matrice diagonale ')
    disp(' Autovalori della matrice:')
    lam=d
    return
end

beta=b.*b;
bb=[b; 0];
xmin=min(d-(abs(bb(1:n))+abs(bb(2:n+1))));
xmax=max(d+(abs(bb(1:n))+abs(bb(2:n+1))));
x0=xmin; xu=xmax;
ind=sort(ind);
for j=length(ind):-1:1
    k=ind(j);
    if k>n
        fprintf('***La matrice non possiede %2d autovalori *** \n',k);
        return
    end
    err=abs(xu-x0)/2;
    it=0;
    while err > toll & it<nmax
        mu=(x0+xu)/2;
        % valutazione dei segni dei polinomi qi nel punto mu
        q=1; a=0;
        for i=1:n
            if q~=0
                s=beta(i)/q;
            else
                s=abs(beta(i))/eps;
            end
            q=(mu-d(i))-s;
            a=a+(q<0);
        end
        % individuazione dell'intervallo a cui appartiene l'autovalore
        if (n-a)>=k
            xu=mu;
        else
            x0=mu;
        end
        lam(j)=mu;  
        err=abs(xu-x0)/2;
        it=it+1;
    end 
    x0=xmin;
    xu=mu;
    if it==nmax
        fprintf('Superato in numero max di iteraz. nel calcolo del %2d autovalore \n',k);
   end
end 
fprintf(' \n Il numero di autovalori trovati sono: %3d \n \n',length(ind));
fprintf('Gli autovalori cercati sono: \n\n');
fprintf( '%22.12e \n',lam);
