
function [xvect,xdiff,fx,it,p,c]=Bisezione(a,b,nmax,toll,fun)

%__________________________________________________________________________
%
% Function: Bisezione
% ___________________
%
%
%    Chiamata: [xvect,xdiff,fx,it,p,c]=Bisezione(a,b,nmax,toll,fun);
% 
%    Commento:
%    Metodo della bisezione per determinare uno zero di un'equazione f(x)=0;
%    
%
%
% Parametri in ingresso:
% ______________________
%
%     a,b        Intervallo in cui si trova lo zero di f.
%     nmax       Numero massimo di iterazioni.
%     toll       Tolleranza sul test d'arresto (|bk-ak|/2<toll con ak e bk 
%                estremi dell'itervallo alla k-esima iterazione).
%     fun        Stringa contenente la funzione a primo membro dell'equazione 
%                f(x)=0.
%
% Parametri in uscita:
% _____________________
%
%     xvect      Vettore colonna  di lunghezza it, contenente le 
%                approssimazioni della soluzione calcolate 
%                dal metodo (l'ultima componente e' la soluzione con la 
%                tolleranza fissata).
%     xdiff      Vettore colonna di lunghezza it, contenente i valori 
%                |b(k)-a(k)|/2.                  
%     fx         Vettore colonna di lunghezza it, contenente le 
%                valutazioni, in modulo, di f in xvect.
%     it         Numero di iterazioni effettuate dal metodo.
%     p          Vettore colonna, di lunghezza it-1, contenente, nell'ultima
%                componente, l'ordine di convergenza stimato da stimap.m.
%     c          Vettore colonna, di dimensioni it-1, contenente, nell'ultima
%                componente, la costante asintotica di convergenza stimata 
%                da stimap.m.
%
%   Functions richieste: Stimap.
%__________________________________________________________________________
%
% Esempio di utilizzo della function applicata alla seguente equazione:
%
% (x^2)/3-sin(x)=0     
% 
% clear all
% clc
%
% %Separazione grafica:
% x=0:0.01:pi;
% y1=x.^2/3;
% y2=sin(x);
% plot(x,y1,x,y2);grid on
% 
% a=1.5;b=2;
% nmax=200;
% toll=1.e-14;
% fun='x.^2/3-sin(x)';
% [xvect,xdiff,fx,it,p,c]=Bisezione(a,b,nmax,toll,fun);
%__________________________________________________________________________


amp=1;it=0;
x=a;
xvect=[];
fx=[];xdiff=[];
p=[];c=[]; 
while it<nmax & amp>toll  
   c=(a+b)/2;
   x=c;
   fc=eval(fun);
   x=a;
   if (fc*eval(fun) > 0)
      a=c;
   else
      b=c;
   end
   xvect=[xvect;c];
   fx=[fx;abs(fc)];
   amp=abs(b-a)/2;
   xdiff=[xdiff;amp];
   it=it+1; 
end

if it>=nmax
    disp('Superato il numero massimo di iterazioni')
end

% Stima dell'ordine del metodo
if it>3
   [p,c]=Stimap(xvect,it); % Il vettore xvect ha lunghezza it
end

fprintf(' \n Numero di Iterazioni : %5.0f \n',it);
fprintf(' Radice calcolata     : %.16e \n',xvect(end)); % xvect(end)=xvect(it);
fprintf(' Ordine stimato       : %.16f \n',p(end)); % p(end)=p(it-1);
fprintf(' Fattore di riduzione : %.16f \n',c(end));% c(end)=c(it-1);

