
function [xvect,xdiff,fx,it,p,c]=Newton_m(x0,nmax,toll,fun,dfun,mol);

%_____________________________________________________________________________
%
% Funzione: Newton_m
% __________________
%
%
%    Chiamata:  [xvect,xdiff,fx,it,p,c]=Newton_m(x0,nmax,toll,fun,dfun,mol);
% 
%    Commento:
%
%    Metodo di Newton modificato per determinare uno zero di molteplicita'
%    m di un'equazione f(x)=0.
%    
%
% Parametri in ingresso:
% ______________________
%
%     x0         Punto di innesco del metodo.
%     nmax       Numero massimo di iterazioni.
%     toll       Tolleranza sul test d'arresto (differenza tra due iterate, 
%                in modulo).
%     fun, dfun  Stringhe contenenti la funzione a primo membro dell'equazione 
%                f(x)=0 e la sua derivata.
%     mol        Presunta molteplicita' della radice cercata.
%
%
% Parametri in uscita:
% _____________________
%
%     xvect      Vettore colonna  di lunghezza it+1, contenente il punto
%                d'innesco e le approssimazioni della soluzione calcolate 
%                dal metodo (l'ultima componente e' la soluzione con la 
%                tolleranza fissata e la prima e' il punto d'innesco).
%     xdiff      Vettore colonna di lunghezza it+1, contenente la 
%                differenza, in modulo, tra due iterate. xdiff(1) e' posto
%                uguale a 0.
%     fx         Vettore colonna di lunghezza it+1, contenente le 
%                valutazioni, in modulo, di f in xvect.
%     it         Numero di iterazioni effettuate dal metodo.
%     p          Vettore colonna, di lunghezza it, contenente, nell'ultima
%                componente, l'ordine di convergenza stimato da stimap.m.
%     c          Vettore colonna, di dimensioni it, contenente, nell'ultima
%                componente, la costante asintotica di convergenza stimata 
%                da stimap.m.
%
%   Functions richieste: Stimap.
%_____________________________________________________________________________
%
% Esempio di utilizzo della function applicata alla seguente equazione:
%
% 2x(1-cos(x^2-2))=0     
% 
% clear all
% clc
% 
% %Separazione grafica:
% x=0:0.01:2;
% y1=2*x;
% y2=2*x.*cos(x.^2-2);
% plot(x,y1,x,y2);grid on
% 
% x0=2;
% nmax=50;
% toll=1.e-14;
% fun='2*x.*(1-cos(x.^2-2))';
% dfun='2-2*cos(x.^2-2)+4*x.^2.*sin(x.^2-2)';
% mol=2;
% xvero=sqrt(2); %soluzione esatta
% [xvect,xdiff,fx,it,p,c]=Newton_m(x0,nmax,toll,fun,dfun,mol);
%_____________________________________________________________________________




diff_x=1;
it=0;
x=x0; fxv=eval(fun);
xvect=x0;
fx=abs(fxv);
xdiff=0;
p=0;c=0;
while it<nmax & diff_x>toll
   x=x0;
   dfx=eval(dfun);
   if abs(dfx)<=eps
	   disp('Arresto per azzeramento di dfun');
       return
   end    
    xn=x0-mol*fxv/dfx;
 	diff_x=abs(xn-x0);
	xdiff=[xdiff;diff_x];
   	x=xn;fxv=eval(fun);
   	xvect=[xvect;xn];
	fx=[fx;abs(fxv)];
	x0=xn;
    it=it+1;
end

if it>=nmax
    disp('Superato il numero massimo di iterazioni')
end

% Stima dell'ordine del metodo
if it+1>3
   [p,c]=Stimap(xvect,it+1);% Il vettore xvect ha lunghezza it+1;
end


fprintf(' \n Numero di Iterazioni : %.0f \n',it);
fprintf(' Radice calcolata     : %.16e \n',xvect(end)); % xvect(end)=xvect(it+1);
fprintf(' Ordine stimato       : %.16f \n',p(end)); % p(end)=p(it); 
fprintf(' Fattore di riduzione : %.16f \n\n',c(end)); % c(end)=c(it);
