
function [xvect,xdiff,fx,it,p,c]=Secanti(x0,x1,nmax,toll,fun)

%_____________________________________________________________________________
%
% Function: Secanti
% _________________
%
%
%    Chiamata:  [xvect,xdiff,fx,it,p,c]=Secanti(x0,x1,nmax,toll,fun);
% 
%    Commento:
%
%    Metodo delle secanti per determinare uno zero di un'equazione f(x)=0;
%    
%
% Parametri in ingresso:
% ______________________
%
%     x0,x1      Punti di innesco del metodo.
%     nmax       Numero massimo di iterazioni.
%     toll       Tolleranza sul test d'arresto (differenza tra due iterate, 
%                in modulo).
%     fun        Stringa contenente la funzione a primo membro dell'equazione 
%                f(x)=0.
%
% Parametri in uscita:
% _____________________
%
%     xvect      Vettore colonna  di lunghezza it+2, contenente i due punti
%                d'innesco e le approssimazioni della soluzione calcolate 
%                dal metodo (l'ultima componente e' la soluzione con la 
%                tolleranza fissata; la prima  e la seconda componente sono
%                i punti d'innesco).
%     xdiff      Vettore colonna di lunghezza it+2, contenente la 
%                differenza, in modulo, tra due iterate. xdiff(1) e' posto
%                uguale a 0.
%     fx         Vettore colonna di lunghezza it+2, contenente le 
%                valutazioni, in modulo, di f in xvect.
%     it         Numero di iterazioni effettuate dal metodo.
%     p          Vettore colonna, di lunghezza it+1, contenente, nell'ultima
%                componente, l'ordine di convergenza stimato da stimap.m.
%     c          Vettore colonna, di dimensioni it+1, contenente, nell'ultima
%                componente, la costante asintotica di convergenza stimata 
%                da stimap.m.
%
% Functions richieste: Stimap.
%_____________________________________________________________________________
%
% Esempio di utilizzo della function applicata alla seguente equazione:
%
% sinh(x)-1/x=0     
% 
%
% clear all
% clc
% 
% % Separazione grafica:
% x=0.5:0.01:1.5;
% y1=1./x;
% y2=sinh(x);
% plot(x,y1,x,y2);grid on
% 
% x0=1.5;x1=1.6;
% nmax=50;
% toll=1.e-14;
% fun='sinh(x)-1./x';
% [xvect,xdiff,fx,it,p,c]=Secanti(x0,x1,nmax,toll,fun);
%____________________________________________________________________________





diff_x=1;
it=0;
x=x0; fx0=eval(fun);
x=x1; fx1=eval(fun);
xvect=[x0;x1];
fx=[abs(fx0);abs(fx1)];
xdiff=[0;abs(x1-x0)];
p=0;c=0;

while it<nmax & diff_x>toll
   if abs(fx1-fx0)<=eps
       disp('Arresto per azzeramento del denominatore')
       return
   end    
   x=x1-fx1*(x1-x0)/(fx1-fx0);
   fxn=eval(fun);
   xvect=[xvect;x];
   fx=[fx;abs(fxn)];
   diff_x=abs(x1-x);
   xdiff=[xdiff;diff_x];
   x0=x1;fx0=fx1;
   x1=x; fx1=fxn;
   it=it+1;
end

if it>=nmax
    disp('Superato il numero massimo di iterazioni')
end

% Stima dell'ordine del metodo
if it+2>3
   [p,c]=Stimap(xvect,it+2);% Il vettore xvect ha lunghezza it+2;
end


fprintf(' \n Numero di Iterazioni : %.0f \n',it);
fprintf(' Radice calcolata     : %.16e \n', xvect(end)); % xvect(end)=xvect(it+2);
fprintf(' Ordine stimato       : %.6f \n',p(end)); % p(end)=p(it+1);
fprintf(' Fattore di riduzione : %.16f \n',c(end)); % c(end)=c(it+1);


