
function [x] = Backsub(U,y)

%___________________________________________________________________
%
% Funzione: Backsub
% _____________________
%
% Chiamata:   [x] = Backsub(U,y);
%
% La function 'Backsub' risolve il sistema lineare Ux=y, con U 
% matrice triangolare superiore e non singolare, attraverso il 
% metodo delle sostituzioni all'indietro.
%
% Parametri in ingresso:
% ______________________
%
%  U   matrice del sistema, di dimensione nxn, triangolare superiore 
%      e non singolare;
%  y   vettore colonna di lunghezza n, contenente il termine noto 
%      del sistema.
%
% Parametri in uscita:
% ____________________
%
%  x   vettore di lunghezza n, contenente la soluzione del sistema.
%
%___________________________________________________________________

n=length(y);
if prod(abs(diag(U)))<1e-14
   disp('Matrice U singolare');
   x=[];
else
   x(n)=y(n)/U(n,n);
   for i=n-1:-1:1
      x(i)=(y(i)-U(i,i+1:n)*(x(i+1:n))')/U(i,i);
   end
   x=x';
end
