
function [x,iter] = Gausspv_r(A,b,toll)

%_________________________________________________________________________
%
% Funzione: Gausspv_r
% ____________________
%
% Chiamata: [x,iter] = Gausspv_r(A,b,toll);
%
% 
% La function 'Gausspv_r' risolve un sistema lineare Ax=b 
% attraverso la fattorizzazione LU della matrice PA, con P 
% matrice di permutazione (pivotazione parziale) e esegue 
% poi il raffinamento iterativo della soluzione.
%
%    Ax=b      PA=LU   =>   Ly=Pb
%                           Ux=y
%
%
% Parametri in ingresso:
% _____________________
%
%  A       matrice del sistema, di dimensione nxn;
%  b       vettore colonna di lunghezza n, contenente il termine 
%          noto del sistema.
%  toll    tolleranza per il test d'arresto del raffinamento iterativo;
%
% Parametri in uscita:
% ____________________
%
%  x     vettore colonna di lunghezza n, contenente la  soluzione 
%        del sistema;
%  iter  numero di iterazioni effettuate nel raffinamento iterativo.
%
%
%
%  La function richiama: lu (intrinseca in Matlab), Raff_iter.
%_________________________________________________________________________
%
% Esempio di utilizzo della function applicata al sistema lineare Ax=b 
% dove
%
% A=     5  -2   0
%       -2   5  -2
%        0  -2   5
%
% b=     3
%        1
%        3
% 
% con soluzione vera:
%
% alpha= 1
%        1
%        1
%_________________________________________________________________________
% 
% clear all; clc
% n=3;
% A=5*eye(n)-2*diag(ones(n-1,1),-1)-2*diag(ones(n-1,1),1)
% b=[3 1 3]'
% toll=1.e-13;
%
% [x,iter] = Gausspv_r(A,b,toll)
%
%_________________________________________________________________________

[L,U,P] = lu(A);
y=L\(P*b);
x=U\y;
[x,iter]=Raff_iter(A,b,L,U,P,x,toll);
