
function [x,iter]=Raff_iter(A,b,L,U,P,x,toll)

%_________________________________________________________________________
%
% Funzione. Raff_iter
% ______________________
%
% Chiamata:    [x,iter]=Raff_iter(A,b,L,U,P,x,toll);
%
% La function 'Raff_iter' esegue il raffinamento iterativo della 
% soluzione di un sistema lineare Ax=b.
%
% Parametri in ingresso:
% ______________________
%
% A           matrice nxn dei coefficienti del sistema Ax=b;
% b           vettore colonna di lunghezza n, contenente il termine 
%             noto del sistema Ax=b;
% L, U, P     matrici, di dimensioni nxn, rispettivamente triangolare
%             inferiore, superiore e matrice di permutazione ottenute
%             dalla fattorizzazione PA=LU;
% x           vettore colonna di lunghezza n contenente la soluzione 
%             da raffinare del sistema lineare;
% toll        tolleranza per il test d'arresto.
%
% Parametri in uscita:
% ____________________
%
% x            vettore colonna di lunghezza n, contenente la soluzione 
%              raffinata del sistema lineare;
% iter         numero di iterazioni effettuate per raggiungere la 
%              tolleranza fissata.
% 
%_________________________________________________________________________


iter=0;
residuo=b-A*x;
while norm(residuo,inf)>toll*norm(b,inf)& iter<100 
   y=L\(P*residuo);
   err=U\y;
   x=x+err;
   residuo=b-A*x;
   iter=iter+1;
end
if iter==100 
   disp('Raggiunto il numero massimo di iterazioni')
end
